/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of mst_rdb
 *
 * mst_rdb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * mst_rdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* rdb_map_cols_arst.c */

#include "mst_rdb.h"

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>


/*

  PROCEDURE
    DataColumnMap_st_comp - compare two DataColumnMap_st structures

  DESCRIPTION
    DataColumnMap_st_comp - compare two DataColumnMap_st structures

  RETURNS
    positive, zero, or negative depending on the alphabetical relation
    between the `column' fields of the datastructures.

  AUTHOR
    Diab Jerius, Richard J. Edgar

*/




static int 
   DataColumnMap_st_comp(
	const void *p, 
	const void *q)
{
    return (((DataColumnMapping_st *)p)->column - 
	((DataColumnMapping_st *)q)->column);
}

/*

  PROCEDURE
    rdb_map_cols_arst - Create a map between user requested columns and those
                   in an /rdb file.  The `arst' signifies that the function
    is called with [1] ARrays of column names and type arguments, and [2]
    expects subsequent reads to go into elements of a STructure.

  DESCRIPTION
    When reading an /rdb file, the user may wish data to be returned in
    an order different from that in the data file.  This routine takes
    a set of column names and creates a map between it and the names
    read from an /rdb header.  The map is used by other /rdb data input
    routines.

  RETURNS
    a pointer to a dynamically allocated DataColumnMap structure
    (filled in with appropriate values).  It prints a message to stderr
    and exits if a user supplied column is not found in the /rdb hdr.
    When the user is finished with the data structures allocated here,
    their memory should be freed with the rdb_free_map() function.

  AUTHOR
    Diab Jerius, Richard J. Edgar

  NOTE
    It is generally cleaner to use rdb_map_cols_stst().

  EXAMPLE
@example
@include lib/mstrdb_c.texi
@end example
    

*/



DataColumnMap_st * rdb_map_cols_arst(
  rdbHeader const *hdr,		/* the rdb header */
  unsigned long ncols,		/* the number of columns to read */
  char *col[],	                /* an array of pointers to strings holding the
				   names of the columns to map */
  RDB_Type const type[],        /* an array of type specifiers; allowed 
				   values are RDB_String, RDB_Num 
                                   (i.e. double) */
  size_t const data_offset[]    /* offsets into struct for data values */
)
{
  DataColumnMap_st *map = NULL;
  unsigned long i;

  tf_enter("rdb_map_cols_arst");

  map = malloc(sizeof(DataColumnMap_st));
  map->map =  malloc(sizeof(DataColumnMapping_st) * ncols);
  if (! map || !map->map)
    tf_exit(ExitERR_alloc, "couldn't allocate %d columns", ncols);

  map->n = ncols;

  for (i = 0; i < ncols ; i++)
  {
    unsigned long j;

    /* search through header column names for match. very inefficient, assumes
       that there are a small number of columns */
    for (j = 0 ; j < hdr->n && strcmp(col[i], hdr->col_name[j]) ; j++);

    if (j == hdr->n)
      tf_exit(ExitERR_illegalArg,
            "no column `%s' in rdb header", col[i]);

    map->map[i].offset = data_offset[i];
    map->map[i].column = j;
    map->map[i].type = type[i];
  }

  /* sort columns so can retrieve in order that they appear in the file */
  qsort(map->map, ncols, sizeof(DataColumnMapping_st), DataColumnMap_st_comp);

  tf_leave();
  return map;
}

